#include "TCPServer.h"
#define DEBUG

TCPServer::TCPServer(unsigned int port, uint8_t maxClient) : _wifiServer(port), _port(port), _serverStarted(true), _maxClient(maxClient), _currentClient(NULL)
{
  _wifiServer.begin();
}

TCPServer::~TCPServer()
{
  _clientList.dispose();
}

void TCPServer::runServer()
{
  handleNewClients();
  getClientData();
}

void TCPServer::handleNewClients()
{
  WiFiClient wc;
  
  if(_maxClient == -1 || _maxClient > _clientList.count())
  {
	wc = _wifiServer.available();
  }
  
  if(wc && wc.connected())
  {
	_clientList.addFirst(new TCPServerHelper::TCPServerClient({wc, TCPServerHelper::TCPServerClient::NEW,{'/0'},0,false,_clientList.count(),{TCPServerHelper::UNDEFINED, TCPServerHelper::UNKNOWN, TCPServerHelper::UNKNOWN_MIME, Dictionary<DictionaryHelper::StringEntity>(), Dictionary<DictionaryHelper::StringEntity>(), NULL,NULL},TCPServerHelper::HTTP_VERB}));
	#ifdef DEBUG
	Serial.print("New client accepted : ");Serial.println(_clientList.count()-1);
	#endif
  }

  _currentClient = _clientList.removeLastRef();//We pick a client in the list to process it's request
  
}

void TCPServer::getClientData()
{
  if(_currentClient == NULL)
  {
    return;
  }
  
  uint32_t bytesAvailable((_currentClient->client).available());
  
  if(bytesAvailable)
  {
	if(_currentClient->dataSize < 255)
	{
		int freeSpace = (255-1/*for \0*/ - _currentClient->dataSize);
		int amountToBeRead = bytesAvailable < freeSpace ? bytesAvailable : freeSpace;//;bytesAvailable < 254 ? bytesAvailable : 254;
	
		_currentClient->client.read(_currentClient->data + _currentClient->dataSize, amountToBeRead);
		_currentClient->dataSize+=amountToBeRead;
		_currentClient->data[_currentClient->dataSize] = '\0';
		_currentClient->newDataAvailable = true;
	}
  }
  else if(!(_currentClient->client).connected())
  {
	#ifdef DEBUG
    Serial.print("Client can be discarded : ");Serial.println(_currentClient->id);
    #endif
    _currentClient->clientState = TCPServerHelper::TCPServerClient::DISCARDED;
  }
  
  
  if(_currentClient->dataSize > 0)
  {
    processClientData(_currentClient);//We process the actual data
    _currentClient->newDataAvailable = false;
  }
  
  if(_currentClient->clientState == TCPServerHelper::TCPServerClient::DISCARDED)
  {
    _currentClient->client.stop();
	#ifdef DEBUG
    Serial.print("Client was discarded : ");Serial.println(_currentClient->id);
    #endif
    delete _currentClient;
    _currentClient = NULL;
    return;
  }
  
  _clientList.addFirst(_currentClient);
  
}

void TCPServer::processClientData(TCPServerHelper::TCPServerClient *client)
{
	Serial.print("Client --> ");Serial.print(client->id);Serial.print(" : ");Serial.println((char *)client->data);
}

uint8_t TCPServer::getMaxClient()
{
  return _maxClient;
}

uint8_t TCPServer::getClientsCount()
{
  return _clientList.count();
}

void TCPServer::startServer()
{
	_serverStarted = true;
}

void TCPServer::stopServer()
{
	_serverStarted = false;
	_clientList.dispose();
}
