#ifndef TCPSERVER_H
#define TCPSERVER_H
#include <Arduino.h>
#include <ESP8266WiFi.h>
#include "List.h"
#include "Dictionary.h"

#define MAX_CLIENT -1

namespace TCPServerHelper
{
  enum HttpRequestMethod {UNDEFINED, GET, HEAD, POST, PUT, DELETE, CONNECT, OPTIONS, TRACE, PATCH};
  enum HttpVersion {UNKNOWN, HTTP_0_9, HTTP_1_1, HTTP_1_0, HTTP_2_0};
  enum HttpMIMEType{UNKNOWN_MIME, TEXT_PLAIN, TEXT_CSS, TEXT_HTML, TEXT_JAVASCRIPT, APPLICATION_JSON, APPLICATION_X_WWW_FORM_URLENCODED, IMAGE_PNG, IMAGE_JPEG, AUDIO_MPEG, APPLICATION_OCTET_STREAM};
  enum HttpParserStatus {HTTP_VERB, HTTP_RESSOURCE, HTTP_VERSION, HTTP_PARAMS, POST_DATA};
  
  struct HttpRequestData
  {
    HttpRequestMethod HRM;
    HttpVersion HV;
    HttpMIMEType HMT;
    Dictionary<DictionaryHelper::StringEntity> getParams;
    Dictionary<DictionaryHelper::StringEntity> postParams;
    char *httpResource;
    char *httpBody;
  };
	
  struct TCPServerClient
  {
    WiFiClient client;
    enum ClientState {NEW, HANDLED, QUERY_PARSED, RESPONSE_SENT, DISCARDED} clientState;
    uint8_t data[255];
	uint16_t dataSize;
    boolean newDataAvailable;
    uint8_t id;
	HttpRequestData httpRequestData;
	HttpParserStatus httpParserState;
	
    bool operator==(TCPServerClient& Object){return this->client == Object.client;}
	void clearHttpRequestData(TCPServerHelper::HttpRequestData *httpRequestData)
	{
	    free(httpRequestData->httpResource);free(httpRequestData->httpBody);
	    httpRequestData->getParams.dispose();
	    httpRequestData->postParams.dispose();
	}
	~TCPServerClient()
	{
		client.stop();
		clearHttpRequestData(&httpRequestData);
	}
	void freeDataBuffer(uint16_t size)
	{
		if(size > 254) size = 254;
		
		uint16_t secureSize = size > dataSize ? dataSize : size;
		strcpy((char *)data, (char *)data + secureSize);
		dataSize -= secureSize;
	}
  };
}

class TCPServer
{
  public: 
    TCPServer(unsigned int port = 80, uint8_t maxClient = MAX_CLIENT);
    ~TCPServer();
    uint8_t getMaxClient();
    uint8_t getClientsCount();
    virtual void runServer();
	virtual void startServer();
	virtual void stopServer();
  protected:
  private:
    virtual void handleNewClients();
    virtual void getClientData();
    virtual void processClientData(TCPServerHelper::TCPServerClient *client);
	
	boolean _serverStarted;
    uint8_t _maxClient;
    unsigned int _port;
    WiFiServer _wifiServer;
    TCPServerHelper::TCPServerClient *_currentClient; //current client to be processed
    List<TCPServerHelper::TCPServerClient> _clientList;
};

#endif //TCPSERVER_H
