#include "TCPWebServer.h"
#define DEBUG

TCPWebServer::TCPWebServer(unsigned int port, uint8_t maxClient) : TCPServer(port, maxClient)
{
	
}

void debugInfo()
{
  uint32_t freeMem;
  uint16_t biggestContigMemBlock;
  uint8_t frag;
  ESP.getHeapStats(&freeMem, &biggestContigMemBlock, &frag);
  Serial.print("Free MEM : ");Serial.println(freeMem);
  Serial.print("Heap Frag : ");Serial.println(frag);
}

void TCPWebServer::processClientData(TCPServerHelper::TCPServerClient *client)
{
	if(client->newDataAvailable)
	{
		switch(client->httpParserState)
		{
			case TCPServerHelper::HTTP_VERB:
			{
				char *pVerb = strstr((char *)client->data, " ");
				
				if(pVerb != NULL)
				{
					*pVerb = '\0';
					client->httpRequestData.HRM = getHttpVerbEnumValue((char *)client->data);
					client->freeDataBuffer((pVerb - (char *)client->data) +1);
					#ifdef DEBUG
					Serial.print("Verb : ");Serial.println(client->httpRequestData.HRM);
					#endif
					client->httpParserState = TCPServerHelper::HttpParserStatus::HTTP_RESSOURCE;
				}
			}	
				break;
			case TCPServerHelper::HTTP_RESSOURCE:
			{
				char *pRsrc = strstr((char *)client->data, " ");
			
				if(pRsrc != NULL)
				{
					*pRsrc = '\0';
					uint16_t length = pRsrc - (char *)client->data;
					client->httpRequestData.httpResource = (char *) malloc(sizeof(char) * length + 1 ); //for \0
					if(client->httpRequestData.httpResource != NULL)
						strcpy(client->httpRequestData.httpResource, (char *)client->data);
					else
					{
						client->clientState = TCPServerHelper::TCPServerClient::ClientState::DISCARDED;
						break;
					}
					
					#ifdef DEBUG
					Serial.print("Resrc : ");Serial.println(client->httpRequestData.httpResource);
					#endif
					client->freeDataBuffer(length + 1);
					
					client->httpParserState = TCPServerHelper::HttpParserStatus::HTTP_VERSION;
				}
				else if(client->dataSize == 254)
				{
					client->httpRequestData.httpResource = (char *) malloc(sizeof(char) * client->dataSize + 1 ); //for \0
					if(client->httpRequestData.httpResource != NULL)
						strcpy(client->httpRequestData.httpResource, (char *)client->data);
					else
					{
						client->clientState = TCPServerHelper::TCPServerClient::ClientState::DISCARDED;
						break;
					}
					
					#ifdef DEBUG
					Serial.print("Resrc : ");Serial.println(client->httpRequestData.httpResource);
					#endif
					client->freeDataBuffer(client->dataSize);
					client->httpParserState = TCPServerHelper::HttpParserStatus::HTTP_VERSION;
				}
			}
				break;
			case TCPServerHelper::HTTP_VERSION:
			{
				char *pEndline = strstr((char *)client->data, "\r\n");
				char *pVers = strstr((char *)client->data, "HTTP/");
				
				if(pEndline != NULL && pVers!= NULL)
				{
					*pEndline = '\0';
					client->httpRequestData.HV = getHttpVersionEnumValue(pVers+5);
					#ifdef DEBUG
					Serial.print("Vers : ");Serial.println(pVers+5);
					Serial.print("Vers : ");Serial.println(client->httpRequestData.HV);
					#endif
					client->freeDataBuffer((pEndline - (char *)client->data)+2);
					client->clientState = TCPServerHelper::TCPServerClient::ClientState::DISCARDED;
				}
				else //If we do not find it, it's probably split at the end of the buffer, so we ignore some of the resource left over
				{
					client->freeDataBuffer(100);
				}
			}
				break;
			/*case TCPServerHelper::HTTP_PARAMS:
				break;
			case TCPServerHelper::POST_DATA:
				break;
			default :*/
		}
		/*Serial.print("WEB Client --> ");Serial.print(client->id);Serial.print(" : ");Serial.println((char *)client->data);
		client->freeDataBuffer(client->dataSize);*/
	}
	
}

TCPServerHelper::HttpRequestMethod TCPWebServer::getHttpVerbEnumValue(const char *parseBuffer)
{
  //UNDEFINED, GET, HEAD, POST, PUT, DELETE, CONNECT, OPTIONS, TRACE, PATCH
  if(strcmp(parseBuffer,"GET") == 0){return TCPServerHelper::HttpRequestMethod::GET;}
  else if(strcmp(parseBuffer,"POST") == 0){return TCPServerHelper::HttpRequestMethod::POST;}
  else if(strcmp(parseBuffer,"HEAD") == 0){return TCPServerHelper::HttpRequestMethod::HEAD;}
  else if(strcmp(parseBuffer,"PUT") == 0){return TCPServerHelper::HttpRequestMethod::PUT;}
  else if(strcmp(parseBuffer,"DELETE") == 0){return TCPServerHelper::HttpRequestMethod::DELETE;}
  else if(strcmp(parseBuffer,"CONNECT") == 0){return TCPServerHelper::HttpRequestMethod::CONNECT;}
  else if(strcmp(parseBuffer,"TRACE") == 0){return TCPServerHelper::HttpRequestMethod::TRACE;}
  else if(strcmp(parseBuffer,"PATCH") == 0){return TCPServerHelper::HttpRequestMethod::PATCH;}
  else if(strcmp(parseBuffer,"OPTIONS") == 0){return TCPServerHelper::HttpRequestMethod::OPTIONS;}
  else
    return TCPServerHelper::HttpRequestMethod::UNDEFINED;
}

TCPServerHelper::HttpVersion TCPWebServer::getHttpVersionEnumValue(const char *parseBuffer)
{
  //HTTP_0_9, HTTP_1_1, HTTP_1_0, HTTP_2_0
  if(strcmp(parseBuffer,"1.1") == 0){return TCPServerHelper::HttpVersion::HTTP_1_1;}
  else if(strcmp(parseBuffer,"2.0") == 0){return TCPServerHelper::HttpVersion::HTTP_2_0;}
  else if(strcmp(parseBuffer,"1.0") == 0){return TCPServerHelper::HttpVersion::HTTP_1_0;}
  else if(strcmp(parseBuffer,"0.9") == 0){return TCPServerHelper::HttpVersion::HTTP_0_9;}
  else
    return TCPServerHelper::HttpVersion::UNKNOWN;
}
